@@ -73,12 +73,14 @@ func (d *decodeGen) assignAndCheck(name string, typ string) {
 }
 
 func (d *decodeGen) structAsTuple(s *Struct) {
-	nfields := len(s.Fields)
-
 	sz := randIdent()
 	d.p.declare(sz, u32)
 	d.assignAndCheck(sz, arrayHeader)
-	d.p.arrayCheck(strconv.Itoa(nfields), sz)
+	if s.AsVarTuple {
+		d.p.printf("\nif %[1]s == 0 { return }", sz)
+	} else {
+		d.p.arrayCheck(strconv.Itoa(len(s.Fields)), sz)
+	}
 	for i := range s.Fields {
 		if !d.p.ok() {
 			return
@@ -98,6 +100,12 @@ func (d *decodeGen) structAsTuple(s *Struct) {
 		if anField {
 			d.p.printf("\n}") // close if statement
 		}
+		if s.AsVarTuple {
+			d.p.printf("\nif %[1]s--; %[1]s == 0 { return }", sz)
+		}
+	}
+	if s.AsVarTuple {
+		d.p.printf("\nfor ; %[1]s > 0; %[1]s-- {\nif err = dc.Skip(); err != nil {\nerr = msgp.WrapError(err)\nreturn\n}\n}", sz)
 	}
 }
 
@@ -218,6 +226,9 @@ func (d *decodeGen) gBase(b *BaseElem) {
 	case Ext:
 		d.p.printf("\nerr = dc.ReadExtension(%s)", vname)
 	default:
+		if b.Value == Time && d.ctx.asUTC {
+			bname += "UTC"
+		}
 		if b.Convert {
 			d.p.printf("\n%s, err = dc.Read%s()", tmp, bname)
 		} else {
@@ -233,7 +244,7 @@ func (d *decodeGen) gBase(b *BaseElem) {
 
 	// close block for 'tmp'
 	if b.Convert && b.Value != IDENT {
-		if b.ShimMode == Cast {
+		if b.ShimMode == Cast && !b.ShimErrs {
 			d.p.printf("\n%s = %s(%s)\n}", vname, b.FromBase(), tmp)
 		} else {
 			d.p.printf("\n%s, err = %s(%s)\n}", vname, b.FromBase(), tmp)
@@ -257,9 +268,8 @@ func (d *decodeGen) gMap(m *Map) {
 	// pair and assign
 	d.needsField()
 	d.p.printf("\nfor %s > 0 {\n%s--", sz, sz)
-	d.p.declare(m.Keyidx, "string")
+	m.readKey(d.ctx, d.p, d, d.assignAndCheck)
 	d.p.declare(m.Validx, m.Value.TypeName())
-	d.assignAndCheck(m.Keyidx, stringTyp)
 	d.ctx.PushVar(m.Keyidx)
 	m.Value.SetIsAllowNil(false)
 	next(d, m.Value)
