@@ -255,11 +255,11 @@ ridx:
 }
 
 func (a *Array) TypeName() string {
-	if a.common.alias != "" {
-		return a.common.alias
+	if a.alias != "" {
+		return a.alias
 	}
-	a.common.Alias(fmt.Sprintf("[%s]%s", a.Size, a.Els.TypeName()))
-	return a.common.alias
+	a.Alias(fmt.Sprintf("[%s]%s", a.Size, a.Els.TypeName()))
+	return a.alias
 }
 
 func (a *Array) Copy() Elem {
@@ -282,10 +282,14 @@ func (a *Array) IfZeroExpr() string { return "" }
 // Map is a map[string]Elem
 type Map struct {
 	common
-	Keyidx     string // key variable name
-	Validx     string // value variable name
-	Value      Elem   // value element
-	isAllowNil bool
+	Keyidx        string // key variable name
+	Validx        string // value variable name
+	Key           Elem   // key element (if not string)
+	Value         Elem   // value element
+	AllowMapShims bool   // Allow map keys to be shimmed (default true)
+	AllowBinMaps  bool   // Allow maps with binary keys to be used (default false)
+	AutoMapShims  bool   // Automatically shim map keys of builtin types(default false)
+	isAllowNil    bool
 }
 
 func (m *Map) SetVarname(s string) {
@@ -303,11 +307,15 @@ ridx:
 }
 
 func (m *Map) TypeName() string {
-	if m.common.alias != "" {
-		return m.common.alias
+	if m.alias != "" {
+		return m.alias
 	}
-	m.common.Alias("map[string]" + m.Value.TypeName())
-	return m.common.alias
+	keyType := "string"
+	if m.Key != nil {
+		keyType = m.Key.TypeName()
+	}
+	m.Alias("map[" + keyType + "]" + m.Value.TypeName())
+	return m.alias
 }
 
 func (m *Map) Copy() Elem {
@@ -316,6 +324,21 @@ func (m *Map) Copy() Elem {
 	return &g
 }
 
+// readKey will read the key into the variable named by m.Keyidx.
+func (m *Map) readKey(ctx *Context, p printer, t traversal, assignAndCheck func(name string, base string)) {
+	if m.Key != nil && m.AllowBinMaps {
+		p.declare(m.Keyidx, m.Key.TypeName())
+		ctx.PushVar(m.Keyidx)
+		m.Key.SetVarname(m.Keyidx)
+		next(t, m.Key)
+		ctx.Pop()
+		return
+	}
+	// No key, so we assume the key as a string.
+	p.declare(m.Keyidx, "string")
+	assignAndCheck(m.Keyidx, stringTyp)
+}
+
 func (m *Map) Complexity() int {
 	// Complexity of maps are considered constant. Children should decide on their own.
 	return 3
@@ -352,11 +375,11 @@ func (s *Slice) SetVarname(a string) {
 }
 
 func (s *Slice) TypeName() string {
-	if s.common.alias != "" {
-		return s.common.alias
+	if s.alias != "" {
+		return s.alias
 	}
-	s.common.Alias("[]" + s.Els.TypeName())
-	return s.common.alias
+	s.Alias("[]" + s.Els.TypeName())
+	return s.alias
 }
 
 func (s *Slice) Copy() Elem {
@@ -429,11 +452,11 @@ func (s *Ptr) SetVarname(a string) {
 }
 
 func (s *Ptr) TypeName() string {
-	if s.common.alias != "" {
-		return s.common.alias
+	if s.alias != "" {
+		return s.alias
 	}
-	s.common.Alias("*" + s.Value.TypeName())
-	return s.common.alias
+	s.Alias("*" + s.Value.TypeName())
+	return s.alias
 }
 
 func (s *Ptr) Copy() Elem {
@@ -459,13 +482,14 @@ func (s *Ptr) IfZeroExpr() string { return s.Varname() + " == nil" }
 
 type Struct struct {
 	common
-	Fields  []StructField // field list
-	AsTuple bool          // write as an array instead of a map
+	Fields     []StructField // field list
+	AsTuple    bool          // write as an array instead of a map
+	AsVarTuple bool          // write as an array of variable length instead of a map
 }
 
 func (s *Struct) TypeName() string {
-	if s.common.alias != "" {
-		return s.common.alias
+	if s.alias != "" {
+		return s.alias
 	}
 	str := "struct{\n"
 	for i := range s.Fields {
@@ -474,8 +498,8 @@ func (s *Struct) TypeName() string {
 			" " + s.Fields[i].RawTag + ";\n"
 	}
 	str += "}"
-	s.common.Alias(str)
-	return s.common.alias
+	s.Alias(str)
+	return s.alias
 }
 
 func (s *Struct) SetVarname(a string) {
@@ -574,8 +598,10 @@ type BaseElem struct {
 	ShimMode     ShimMode  // Method used to shim
 	ShimToBase   string    // shim to base type, or empty
 	ShimFromBase string    // shim from base type, or empty
+	ShimErrs     bool      // ShimToBase has errors on function
 	Value        Primitive // Type of element
 	Convert      bool      // should we do an explicit conversion?
+	zerocopy     bool      // Allow zerocopy for byte slices in unmarshal.
 	mustinline   bool      // must inline; not printable
 	needsref     bool      // needs reference for shim
 	allowNil     *bool     // Override from parent.
@@ -624,11 +650,11 @@ func (s *BaseElem) SetVarname(a string) {
 // TypeName returns the syntactically correct Go
 // type name for the base element.
 func (s *BaseElem) TypeName() string {
-	if s.common.alias != "" {
-		return s.common.alias
+	if s.alias != "" {
+		return s.alias
 	}
 	s.common.Alias(s.BaseType())
-	return s.common.alias
+	return s.alias
 }
 
 // ToBase, used if Convert==true, is used as tmp = {{ToBase}}({{Varname}})
@@ -752,7 +778,8 @@ func (s *BaseElem) ZeroExpr() string {
 		return "(time.Time{})"
 	case JsonNumber:
 		return `""`
-
+	case Intf:
+		return "nil"
 	}
 
 	return ""
