@@ -28,6 +28,13 @@ func (e *encodeGen) Apply(dirs []string) error {
 }
 
 func (e *encodeGen) writeAndCheck(typ string, argfmt string, arg interface{}) {
+	if e.ctx.compFloats && typ == "Float64" {
+		typ = "Float"
+	}
+	if e.ctx.newTime && typ == "Time" {
+		typ = "TimeExt"
+	}
+
 	e.p.printf("\nerr = en.Write%s(%s)", typ, fmt.Sprintf(argfmt, arg))
 	e.p.wrapErrCheck(e.ctx.ArgsStr())
 }
@@ -47,7 +54,8 @@ func (e *encodeGen) Fuse(b []byte) {
 	}
 }
 
-func (e *encodeGen) Execute(p Elem) error {
+func (e *encodeGen) Execute(p Elem, ctx Context) error {
+	e.ctx = &ctx
 	if !e.p.ok() {
 		return e.p.err
 	}
@@ -59,12 +67,17 @@ func (e *encodeGen) Execute(p Elem) error {
 		return nil
 	}
 
-	e.ctx = &Context{}
-
 	e.p.comment("EncodeMsg implements msgp.Encodable")
-
-	e.p.printf("\nfunc (%s %s) EncodeMsg(en *msgp.Writer) (err error) {", p.Varname(), imutMethodReceiver(p))
+	rcv := imutMethodReceiver(p)
+	ogVar := p.Varname()
+	if p.AlwaysPtr(nil) {
+		rcv = methodReceiver(p)
+	}
+	e.p.printf("\nfunc (%s %s) EncodeMsg(en *msgp.Writer) (err error) {", ogVar, rcv)
 	next(e, p)
+	if p.AlwaysPtr(nil) {
+		p.SetVarname(ogVar)
+	}
 	e.p.nakedReturn()
 	return e.p.err
 }
@@ -85,9 +98,7 @@ func (e *encodeGen) tuple(s *Struct) {
 	data := msgp.AppendArrayHeader(nil, uint32(nfields))
 	e.p.printf("\n// array header, size %d", nfields)
 	e.Fuse(data)
-	if len(s.Fields) == 0 {
-		e.fuseHook()
-	}
+	e.fuseHook()
 	for i := range s.Fields {
 		if !e.p.ok() {
 			return
@@ -132,6 +143,7 @@ func (e *encodeGen) structmap(s *Struct) {
 
 	omitempty := s.AnyHasTagPart("omitempty")
 	omitzero := s.AnyHasTagPart("omitzero")
+	var closeZero bool
 	var fieldNVar string
 	if omitempty || omitzero {
 
@@ -165,9 +177,11 @@ func (e *encodeGen) structmap(s *Struct) {
 			return
 		}
 
-		// quick return for the case where the entire thing is empty, but only at the top level
-		if !strings.Contains(s.Varname(), ".") {
-			e.p.printf("\nif %s == 0 { return }", fieldNVar)
+		// Skip block, if no fields are set.
+		if nfields > 1 {
+			e.p.printf("\n\n// skip if no fields are to be emitted")
+			e.p.printf("\nif %s != 0 {", fieldNVar)
+			closeZero = true
 		}
 
 	} else {
@@ -216,7 +230,9 @@ func (e *encodeGen) structmap(s *Struct) {
 		if oeField || anField {
 			e.p.print("\n}") // close if statement
 		}
-
+	}
+	if closeZero {
+		e.p.printf("\n}") // close if statement
 	}
 }
 
@@ -229,8 +245,30 @@ func (e *encodeGen) gMap(m *Map) {
 	e.writeAndCheck(mapHeader, lenAsUint32, vname)
 
 	e.p.printf("\nfor %s, %s := range %s {", m.Keyidx, m.Validx, vname)
-	e.writeAndCheck(stringTyp, literalFmt, m.Keyidx)
+	if m.Key != nil {
+		if m.AllowBinMaps {
+			e.ctx.PushVar(m.Keyidx)
+			m.Key.SetVarname(m.Keyidx)
+			next(e, m.Key)
+			e.ctx.Pop()
+		} else {
+			keyIdx := m.Keyidx
+			if key, ok := m.Key.(*BaseElem); ok {
+				if m.AutoMapShims && CanAutoShim[key.Value] {
+					keyIdx = fmt.Sprintf("msgp.AutoShim{}.%sString(%s(%s))", key.Value.String(), strings.ToLower(key.Value.String()), keyIdx)
+				} else if key.Value == String {
+					keyIdx = fmt.Sprintf("%s(%s)", key.ToBase(), keyIdx)
+				} else if key.alias != "" {
+					keyIdx = fmt.Sprintf("string(%s)", keyIdx)
+				}
+			}
+			e.writeAndCheck(stringTyp, literalFmt, keyIdx)
+		}
+	} else {
+		e.writeAndCheck(stringTyp, literalFmt, m.Keyidx)
+	}
 	e.ctx.PushVar(m.Keyidx)
+	m.Value.SetIsAllowNil(false)
 	next(e, m.Value)
 	e.ctx.Pop()
 	e.p.closeblock()
