@@ -28,8 +28,9 @@ func (u *unmarshalGen) needsField() {
 	u.hasfield = true
 }
 
-func (u *unmarshalGen) Execute(p Elem) error {
+func (u *unmarshalGen) Execute(p Elem, ctx Context) error {
 	u.hasfield = false
+	u.ctx = &ctx
 	if !u.p.ok() {
 		return u.p.err
 	}
@@ -41,8 +42,6 @@ func (u *unmarshalGen) Execute(p Elem) error {
 		return nil
 	}
 
-	u.ctx = &Context{}
-
 	u.p.comment("UnmarshalMsg implements msgp.Unmarshaler")
 
 	u.p.printf("\nfunc (%s %s) UnmarshalMsg(bts []byte) (o []byte, err error) {", p.Varname(), methodReceiver(p))
@@ -78,7 +77,11 @@ func (u *unmarshalGen) tuple(s *Struct) {
 	sz := randIdent()
 	u.p.declare(sz, u32)
 	u.assignAndCheck(sz, arrayHeader)
-	u.p.arrayCheck(strconv.Itoa(len(s.Fields)), sz)
+	if s.AsVarTuple {
+		u.p.printf("\nif %[1]s == 0 {\no = bts\nreturn\n}", sz)
+	} else {
+		u.p.arrayCheck(strconv.Itoa(len(s.Fields)), sz)
+	}
 	for i := range s.Fields {
 		if !u.p.ok() {
 			return
@@ -90,11 +93,40 @@ func (u *unmarshalGen) tuple(s *Struct) {
 			u.p.printf("\nif msgp.IsNil(bts) {\nbts = bts[1:]\n%s = nil\n} else {", fieldElem.Varname())
 		}
 		SetIsAllowNil(fieldElem, anField)
+		if s.Fields[i].HasTagPart("zerocopy") {
+			setRecursiveZC(fieldElem, true)
+		}
 		next(u, fieldElem)
+		if s.Fields[i].HasTagPart("zerocopy") {
+			setRecursiveZC(fieldElem, false)
+		}
+
 		u.ctx.Pop()
 		if anField {
 			u.p.printf("\n}")
 		}
+		if s.AsVarTuple {
+			u.p.printf("\nif %[1]s--; %[1]s == 0 {\no = bts\nreturn\n}", sz)
+		}
+	}
+	if s.AsVarTuple {
+		u.p.printf("\nfor ; %[1]s > 0; %[1]s-- {\nbts, err = msgp.Skip(bts)\nif err != nil {\nerr = msgp.WrapError(err)\nreturn\n}\n}", sz)
+	}
+}
+
+// setRecursiveZC will alloc zerocopy for byte fields that are present.
+func setRecursiveZC(e Elem, enable bool) {
+	if base, ok := e.(*BaseElem); ok {
+		base.zerocopy = enable
+	}
+	if el, ok := e.(*Slice); ok {
+		setRecursiveZC(el.Els, enable)
+	}
+	if el, ok := e.(*Array); ok {
+		setRecursiveZC(el.Els, enable)
+	}
+	if el, ok := e.(*Map); ok {
+		setRecursiveZC(el.Value, enable)
 	}
 }
 
@@ -104,6 +136,22 @@ func (u *unmarshalGen) mapstruct(s *Struct) {
 	u.p.declare(sz, u32)
 	u.assignAndCheck(sz, mapHeader)
 
+	oeCount := s.CountFieldTagPart("omitempty") + s.CountFieldTagPart("omitzero")
+	if !u.ctx.clearOmitted {
+		oeCount = 0
+	}
+	bm := bmask{
+		bitlen:  oeCount,
+		varname: sz + "Mask",
+	}
+	if oeCount > 0 {
+		// Declare mask
+		u.p.printf("\n%s", bm.typeDecl())
+		u.p.printf("\n_ = %s", bm.varname)
+	}
+	// Index to field idx of each emitted
+	oeEmittedIdx := []int{}
+
 	u.p.printf("\nfor %s > 0 {", sz)
 	u.p.printf("\n%s--; field, bts, err = msgp.ReadMapKeyZC(bts)", sz)
 	u.p.wrapErrCheck(u.ctx.ArgsStr())
@@ -112,7 +160,7 @@ func (u *unmarshalGen) mapstruct(s *Struct) {
 		if !u.p.ok() {
 			return
 		}
-		u.p.printf("\ncase \"%s\":", s.Fields[i].FieldTag)
+		u.p.printf("\ncase %q:", s.Fields[i].FieldTag)
 		u.ctx.PushString(s.Fields[i].FieldName)
 
 		fieldElem := s.Fields[i].FieldElem
@@ -121,8 +169,18 @@ func (u *unmarshalGen) mapstruct(s *Struct) {
 			u.p.printf("\nif msgp.IsNil(bts) {\nbts = bts[1:]\n%s = nil\n} else {", fieldElem.Varname())
 		}
 		SetIsAllowNil(fieldElem, anField)
+		if s.Fields[i].HasTagPart("zerocopy") {
+			setRecursiveZC(fieldElem, true)
+		}
 		next(u, fieldElem)
+		if s.Fields[i].HasTagPart("zerocopy") {
+			setRecursiveZC(fieldElem, false)
+		}
 		u.ctx.Pop()
+		if oeCount > 0 && (s.Fields[i].HasTagPart("omitempty") || s.Fields[i].HasTagPart("omitzero")) {
+			u.p.printf("\n%s", bm.setStmt(len(oeEmittedIdx)))
+			oeEmittedIdx = append(oeEmittedIdx, i)
+		}
 		if anField {
 			u.p.printf("\n}")
 		}
@@ -130,6 +188,27 @@ func (u *unmarshalGen) mapstruct(s *Struct) {
 	u.p.print("\ndefault:\nbts, err = msgp.Skip(bts)")
 	u.p.wrapErrCheck(u.ctx.ArgsStr())
 	u.p.print("\n}\n}") // close switch and for loop
+	if oeCount > 0 {
+		u.p.printf("\n// Clear omitted fields.\n")
+		if bm.bitlen > 1 {
+			u.p.printf("if %s {\n", bm.notAllSet())
+		}
+		for bitIdx, fieldIdx := range oeEmittedIdx {
+			fieldElem := s.Fields[fieldIdx].FieldElem
+
+			u.p.printf("if %s == 0 {\n", bm.readExpr(bitIdx))
+			fze := fieldElem.ZeroExpr()
+			if fze != "" {
+				u.p.printf("%s = %s\n", fieldElem.Varname(), fze)
+			} else {
+				u.p.printf("%s = %s{}\n", fieldElem.Varname(), fieldElem.TypeName())
+			}
+			u.p.printf("}\n")
+		}
+		if bm.bitlen > 1 {
+			u.p.printf("}")
+		}
+	}
 }
 
 func (u *unmarshalGen) gBase(b *BaseElem) {
@@ -148,7 +227,11 @@ func (u *unmarshalGen) gBase(b *BaseElem) {
 
 	switch b.Value {
 	case Bytes:
-		u.p.printf("\n%s, bts, err = msgp.ReadBytesBytes(bts, %s)", refname, lowered)
+		if b.zerocopy {
+			u.p.printf("\n%s, bts, err = msgp.ReadBytesZC(bts)", refname)
+		} else {
+			u.p.printf("\n%s, bts, err = msgp.ReadBytesBytes(bts, %s)", refname, lowered)
+		}
 	case Ext:
 		u.p.printf("\nbts, err = msgp.ReadExtensionBytes(bts, %s)", lowered)
 	case IDENT:
@@ -156,14 +239,25 @@ func (u *unmarshalGen) gBase(b *BaseElem) {
 			lowered = b.ToBase() + "(" + lowered + ")"
 		}
 		u.p.printf("\nbts, err = %s.UnmarshalMsg(bts)", lowered)
+	case Time:
+		if u.ctx.asUTC {
+			u.p.printf("\n%s, bts, err = msgp.Read%sUTCBytes(bts)", refname, b.BaseName())
+		} else {
+			u.p.printf("\n%s, bts, err = msgp.Read%sBytes(bts)", refname, b.BaseName())
+		}
 	default:
 		u.p.printf("\n%s, bts, err = msgp.Read%sBytes(bts)", refname, b.BaseName())
 	}
 	u.p.wrapErrCheck(u.ctx.ArgsStr())
 
+	if b.Value == Bytes && b.AllowNil() {
+		// Ensure that 0 sized slices are allocated.
+		u.p.printf("\nif %s == nil {\n%s = make([]byte, 0)\n}", refname, refname)
+	}
+
 	// close 'tmp' block
 	if b.Convert && b.Value != IDENT {
-		if b.ShimMode == Cast {
+		if b.ShimMode == Cast && !b.ShimErrs {
 			u.p.printf("\n%s = %s(%s)\n", b.Varname(), b.FromBase(), refname)
 		} else {
 			u.p.printf("\n%s, err = %s(%s)\n", b.Varname(), b.FromBase(), refname)
@@ -225,9 +319,10 @@ func (u *unmarshalGen) gMap(m *Map) {
 
 	// loop and get key,value
 	u.p.printf("\nfor %s > 0 {", sz)
-	u.p.printf("\nvar %s string; var %s %s; %s--", m.Keyidx, m.Validx, m.Value.TypeName(), sz)
-	u.assignAndCheck(m.Keyidx, stringTyp)
+	u.p.printf("\nvar %s %s; %s--", m.Validx, m.Value.TypeName(), sz)
+	m.readKey(u.ctx, u.p, u, u.assignAndCheck)
 	u.ctx.PushVar(m.Keyidx)
+	m.Value.SetIsAllowNil(false)
 	next(u, m.Value)
 	u.ctx.Pop()
 	u.p.mapAssign(m)
