@@ -24,8 +24,13 @@ type FileSet struct {
 	CompactFloats bool                // Use smaller floats when feasible
 	ClearOmitted  bool                // Set omitted fields to zero value
 	NewTime       bool                // Set to use -1 extension for time.Time
-	tagName       string              // tag to read field names from
-	pointerRcv    bool                // generate with pointer receivers.
+	AsUTC         bool                // Set timezone to UTC instead of local
+	AllowMapShims bool                // Allow map keys to be shimmed (default true)
+	AllowBinMaps  bool                // Allow maps with binary keys to be used (default false)
+	AutoMapShims  bool                // Automatically shim map keys of builtin types(default false)
+
+	tagName    string // tag to read field names from
+	pointerRcv bool   // generate with pointer receivers.
 }
 
 // File parses a file at the relative path
@@ -34,12 +39,13 @@ type FileSet struct {
 // directory will be parsed.
 // If unexport is false, only exported identifiers are included in the FileSet.
 // If the resulting FileSet would be empty, an error is returned.
-func File(name string, unexported bool) (*FileSet, error) {
+func File(name string, unexported bool, directives []string) (*FileSet, error) {
 	pushstate(name)
 	defer popstate()
 	fs := &FileSet{
 		Specs:      make(map[string]ast.Expr),
 		Identities: make(map[string]gen.Elem),
+		Directives: append([]string{}, directives...),
 	}
 
 	fset := token.NewFileSet()
@@ -76,7 +82,7 @@ func File(name string, unexported bool) (*FileSet, error) {
 			return nil, err
 		}
 		fs.Package = f.Name.Name
-		fs.Directives = yieldComments(f.Comments)
+		fs.Directives = append(fs.Directives, yieldComments(f.Comments)...)
 		if !unexported {
 			ast.FileExports(f)
 		}
@@ -98,14 +104,14 @@ func File(name string, unexported bool) (*FileSet, error) {
 // applyDirectives applies all of the directives that
 // are known to the parser. additional method-specific
 // directives remain in f.Directives
-func (f *FileSet) applyDirectives() {
-	newdirs := make([]string, 0, len(f.Directives))
-	for _, d := range f.Directives {
+func (fs *FileSet) applyDirectives() {
+	newdirs := make([]string, 0, len(fs.Directives))
+	for _, d := range fs.Directives {
 		chunks := strings.Split(d, " ")
 		if len(chunks) > 0 {
 			if fn, ok := directives[chunks[0]]; ok {
 				pushstate(chunks[0])
-				err := fn(chunks, f)
+				err := fn(chunks, fs)
 				if err != nil {
 					warnf("directive error: %s", err)
 				}
@@ -115,21 +121,21 @@ func (f *FileSet) applyDirectives() {
 			}
 		}
 	}
-	f.Directives = newdirs
+	fs.Directives = newdirs
 }
 
 // applyEarlyDirectives applies all early directives needed before process() is called.
 // additional directives remain in f.Directives for future processing
-func (f *FileSet) applyEarlyDirectives() {
-	newdirs := make([]string, 0, len(f.Directives))
-	for _, d := range f.Directives {
+func (fs *FileSet) applyEarlyDirectives() {
+	newdirs := make([]string, 0, len(fs.Directives))
+	for _, d := range fs.Directives {
 		parts := strings.Split(d, " ")
 		if len(parts) == 0 {
 			continue
 		}
 		if fn, ok := earlyDirectives[parts[0]]; ok {
 			pushstate(parts[0])
-			err := fn(parts, f)
+			err := fn(parts, fs)
 			if err != nil {
 				warnf("early directive error: %s", err)
 			}
@@ -138,7 +144,7 @@ func (f *FileSet) applyEarlyDirectives() {
 			newdirs = append(newdirs, d)
 		}
 	}
-	f.Directives = newdirs
+	fs.Directives = newdirs
 }
 
 // A linkset is a graph of unresolved
@@ -164,12 +170,12 @@ func (f *FileSet) applyEarlyDirectives() {
 // figuring out that D is just a uint64.
 type linkset map[string]*gen.BaseElem
 
-func (f *FileSet) resolve(ls linkset) {
+func (fs *FileSet) resolve(ls linkset) {
 	progress := true
 	for progress && len(ls) > 0 {
 		progress = false
 		for name, elem := range ls {
-			real, ok := f.Identities[elem.TypeName()]
+			real, ok := fs.Identities[elem.TypeName()]
 			if ok {
 				// copy the old type descriptor,
 				// alias it to the new value,
@@ -178,7 +184,7 @@ func (f *FileSet) resolve(ls linkset) {
 				progress = true
 				nt := real.Copy()
 				nt.Alias(name)
-				f.Identities[name] = nt
+				fs.Identities[name] = nt
 				delete(ls, name)
 			}
 		}
@@ -192,18 +198,18 @@ func (f *FileSet) resolve(ls linkset) {
 
 // process takes the contents of f.Specs and
 // uses them to populate f.Identities
-func (f *FileSet) process() {
+func (fs *FileSet) process() {
 	deferred := make(linkset)
 parse:
-	for name, def := range f.Specs {
+	for name, def := range fs.Specs {
 		pushstate(name)
-		el := f.parseExpr(def)
+		el := fs.parseExpr(def)
 		if el == nil {
 			warnf("failed to parse")
 			popstate()
 			continue parse
 		}
-		el.AlwaysPtr(&f.pointerRcv)
+		el.AlwaysPtr(&fs.pointerRcv)
 		// push unresolved identities into
 		// the graph of links and resolve after
 		// we've handled every possible named type.
@@ -213,12 +219,12 @@ parse:
 			continue parse
 		}
 		el.Alias(name)
-		f.Identities[name] = el
+		fs.Identities[name] = el
 		popstate()
 	}
 
 	if len(deferred) > 0 {
-		f.resolve(deferred)
+		fs.resolve(deferred)
 	}
 }
 
@@ -241,13 +247,13 @@ func strToMethod(s string) gen.Method {
 	}
 }
 
-func (f *FileSet) applyDirs(p *gen.Printer) {
+func (fs *FileSet) applyDirs(p *gen.Printer) {
 	// apply directives of the form
 	//
 	// 	//msgp:encode ignore {{TypeName}}
 	//
 loop:
-	for _, d := range f.Directives {
+	for _, d := range fs.Directives {
 		chunks := strings.Split(d, " ")
 		if len(chunks) > 1 {
 			for i := range chunks {
@@ -272,20 +278,21 @@ loop:
 			warnf("empty directive: %q\n", d)
 		}
 	}
-	p.CompactFloats = f.CompactFloats
-	p.ClearOmitted = f.ClearOmitted
-	p.NewTime = f.NewTime
+	p.CompactFloats = fs.CompactFloats
+	p.ClearOmitted = fs.ClearOmitted
+	p.NewTime = fs.NewTime
+	p.AsUTC = fs.AsUTC
 }
 
-func (f *FileSet) PrintTo(p *gen.Printer) error {
-	f.applyDirs(p)
-	names := make([]string, 0, len(f.Identities))
-	for name := range f.Identities {
+func (fs *FileSet) PrintTo(p *gen.Printer) error {
+	fs.applyDirs(p)
+	names := make([]string, 0, len(fs.Identities))
+	for name := range fs.Identities {
 		names = append(names, name)
 	}
 	sort.Strings(names)
 	for _, name := range names {
-		el := f.Identities[name]
+		el := fs.Identities[name]
 		el.SetVarname("z")
 		pushstate(el.TypeName())
 		err := p.Print(el)
@@ -505,6 +512,8 @@ func stringify(e ast.Expr) string {
 		if e.Methods == nil || e.Methods.NumFields() == 0 {
 			return "interface{}"
 		}
+	case *ast.BasicLit:
+		return e.Value
 	}
 	return "<BAD>"
 }
@@ -522,10 +531,54 @@ func (fs *FileSet) parseExpr(e ast.Expr) gen.Elem {
 	switch e := e.(type) {
 
 	case *ast.MapType:
-		if k, ok := e.Key.(*ast.Ident); ok && k.Name == "string" {
-			if in := fs.parseExpr(e.Value); in != nil {
-				return &gen.Map{Value: in}
+		switch k := e.Key.(type) {
+		case *ast.Ident:
+			switch k.Name {
+			case "string":
+				if in := fs.parseExpr(e.Value); in != nil {
+					return &gen.Map{Value: in, AllowBinMaps: fs.AllowBinMaps, AllowMapShims: fs.AllowMapShims, AutoMapShims: fs.AutoMapShims}
+				}
+				warnf("%s: map keys of type  are not supported\n", stringify(e.Key))
+			default:
+				if !fs.AllowMapShims && !fs.AllowBinMaps && !fs.AutoMapShims {
+					warnf("map keys of type %s are not supported without binary keys or shimming\n", stringify(e.Key))
+					return nil
+				}
+				// Allow for other types, assuming they will be shimmed later.
+				key := fs.parseExpr(k)
+				// Types that aren't idents are native types and cannot currently be used as map keys.
+				switch k := key.(type) {
+				case *gen.BaseElem:
+					switch k.Value {
+					case gen.IDENT:
+						if in := fs.parseExpr(e.Value); in != nil {
+							return &gen.Map{Value: in, Key: key, AllowBinMaps: fs.AllowBinMaps, AllowMapShims: fs.AllowMapShims, AutoMapShims: fs.AutoMapShims}
+						}
+						warnf("map keys of type %s are not supported\n", k.TypeName())
+						// Exclude types that cannot be used as native map keys.
+					case gen.Bytes:
+						warnf("map keys of type %s are not supported\n", k.TypeName())
+					default:
+						if in := fs.parseExpr(e.Value); (fs.AllowBinMaps || (fs.AutoMapShims && gen.CanAutoShim[k.Value])) && in != nil {
+							return &gen.Map{Value: in, Key: key, AllowBinMaps: fs.AllowBinMaps, AllowMapShims: fs.AllowMapShims, AutoMapShims: fs.AutoMapShims}
+						}
+						warnf("map keys of type %s are not supported without binary keys or shimming\n", k.TypeName())
+					}
+				default:
+					warnf("map keys of type %s are not supported\n", k.TypeName())
+				}
+				return nil
 			}
+		case *ast.ArrayType:
+			if fs.AllowBinMaps {
+				key := fs.parseExpr(k)
+				if in := fs.parseExpr(e.Value); fs.AllowBinMaps && in != nil {
+					return &gen.Map{Value: in, Key: key, AllowBinMaps: fs.AllowBinMaps, AllowMapShims: fs.AllowMapShims, AutoMapShims: fs.AutoMapShims}
+				}
+			}
+			warnf("array map keys (type %s) are not supported without binary keys or shimming\n", stringify(e.Key))
+		default:
+			warnf("array map key type not supported\n")
 		}
 		return nil
 
@@ -537,7 +590,7 @@ func (fs *FileSet) parseExpr(e ast.Expr) gen.Elem {
 		// everything else.
 		if b.Value == gen.IDENT {
 			if _, ok := fs.Specs[e.Name]; !ok {
-				warnf("non-local identifier: %s\n", e.Name)
+				warnf("possible non-local identifier: %s\n", e.Name)
 			}
 		}
 		return b
